/****************************************************************************
*	EK's ITALIAN parking                                                     *
*	        																                   *
*	17.04.09 game.h             										                *
* 	        																                   *
****************************************************************************/
#include <gccore.h>
#include "..\include\framework.h"
#include "..\include\TextObj.h"
#include "..\include\MyModels.h"
#include "..\include\Spline.h"
#include <wiiuse/wpad.h>

#pragma once

#define  BLOCK_2_STD    0
#define  BLOCK_3_STD    1
#define  BLOCK_2_GOAL   2

struct TGameElem {
   int   nType ;
   int   nLen ;
   int   nDir ;
   CGraphicObject* pGr ;
} ;

struct TUndo {
   CGraphicObject* pGr ;
   Vector          ptOri ;
   Vector          ptEnd ;
} ;

struct TExit {
   int x,y ;
} ;

typedef std::list<TGameElem> 	CListGameElem ;
typedef std::list<TUndo> 	   CListUndo ;
typedef std::list<TExit> 	   CListExit ;

class CGame
{
   public:
      CGame() ;
      ~CGame() ;
      void  FreeAll() ;

      void  SetGrid( int x, int y) ;
      void  AddExit( int x, int y) ;
      void  CreateScene() ;
      void  AddElement( int nType, int nX, int nY, bool bVert) ;
      bool  IsNotEnd() ;
      void  MainCycle( ir_t&  ir) ;
      int  ParseLevel( int nLev, char* szString, u32 size, bool checkOnlyExist) ;      

   private :

      void  ManageRotation( ir_t&  ir) ;
      void  ManageDrag( ir_t&  ir) ;
      u8    GetFlagPos( int x, int y) ;
      void  SetFlagPos( int x, int y, u8 nFlag) ;

     // free/Busy  flags
      void  GetIntPos( TGameElem& elem, int* x, int* y) ;
      void  AttachDetachElem( TGameElem& elem, u8 nAttach) ;
      void  GetValidMovement( TGameElem& elem, float* dMin, float* dMax) ;

      void  Rumble( int nRumble) ;
      void  UpdateRumble( void) ;
      void  BeginDrag( TGameElem& tElem, Vector& ptP) ;
      void  ClampDrag( float& x) ;
      void  Drag( Vector& ptNew) ;
      void  ReleaseDrag( void) ;

      void  AddMoveToUndo( TUndo& undo) ;
      void  Undo( void) ;
      void  UpdateTextMove( void) ;
      void  TestEndGame( void) ;

      void WriteResult( int nLevel) ;
      int ReadResult( int nLevel) ; 

      void Intro() ;
      void Outro() ;



   private:
      int            m_nExpectedMoves ;
      int            m_nStateGame ;
      int            m_nTime ;
      int            m_nWidth ;
      int            m_nHeight ;
      u8*            m_aFlags ;
      CListGameElem  m_lElem ;
      CListUndo      m_lUndo ;
      CListExit      m_lExits ;
      TUndo          m_currUndo ;
      Vector         m_CarPos ;
      
      bool           m_bDragging ;
	   float          m_dDragMin, m_dDragMax ;
	   int            m_nRumble ;
	   Vector         m_ptLastDrag ;
	   TGameElem      m_cCurrSel ;
      
      CTitle*        m_pTitle ;
      CTextObj*      m_pnMoves ;
      CSpline        m_EyePos ;
      CSpline        m_EyeFocus ;
} ;

class CChooser
{
   public :
      CChooser(int nLevels) ;
      
      int GetChoose() ;   
} ;
