/****************************************************************************
*	EK's ITALIAN parking                                                     *
*	        																                   *
*	17.04.09 FrameWork.cpp             										          *
* 	        																                   *
****************************************************************************/
#include "..\include\framework.h"
#include "libpng/pngu/pngu.h"
#include <wiiuse/wpad.h>
#include <malloc.h>
#include <fat.h>
#include <gccore.h>

#define SET_VECTOR( v, x1, y1, z1) { v.x = x1 ; v.y = y1 ; v.z = z1 ;}

//--------------------------------------------------------------------
CFrameWork	  theFrameWork ;
CFrameWork*	  CFrameWork::s_SingleInstance = &theFrameWork ;

// constant
Vector stdAxisX = {1,0,0};
Vector stdAxisY = {0,1,0};
Vector stdAxisZ = {0,0,1};

u8 colors[] ATTRIBUTE_ALIGN(32) = {
	// r, g, b, a
	255, 255, 255, 255,	
	224, 224, 224, 240,
	224, 224, 224, 224,
	192, 192, 192, 206,	
	192, 192, 192, 192,	
	160, 160, 160, 176,	
	160, 160, 160, 160,	
	128, 128, 128, 144,	
	128, 128, 128, 128,	
   112, 112, 112, 112, 
    96,  96,  96,  96,
    80,  80,  80,  80,
    64,  64,  64,  64,
    48,  48,  48,  48,
    32,  32,  32,  32,
    16,  16,  16,  16,
     0,   0,   0,   0
};               

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//----------  FRAMEWORK  ---------------------------------------------
//-------------------------------------------------------------------- 
CFrameWork::CFrameWork()
{
	m_nTexture = 0 ;
	xfb[0] = NULL ;
	xfb[1] = NULL ; 	// Double buffered
	whichfb = 0; 		// Switch
	m_pPointer = NULL ;
}

//--------------------------------------------------------------------
void 
CFrameWork::InitVideo( void) 
{
	VIDEO_Init();
	vmode = VIDEO_GetPreferredMode(NULL); // get default video mode

	// widescreen fix
	if(CONF_GetAspectRatio() == CONF_ASPECT_16_9)
	{
		vmode->viWidth = VI_MAX_WIDTH_PAL-12;
		vmode->viXOrigin = ((VI_MAX_WIDTH_PAL - vmode->viWidth) / 2) + 2;
	}

	VIDEO_Configure (vmode);

	screenheight = 480;
	screenwidth = vmode->fbWidth;

	// Allocate the video buffers
	xfb[0] = (u32 *) MEM_K0_TO_K1 (SYS_AllocateFramebuffer (vmode));
	xfb[1] = (u32 *) MEM_K0_TO_K1 (SYS_AllocateFramebuffer (vmode));

	// A console is always useful while debugging
	console_init (xfb[0], 100, 100, vmode->fbWidth, vmode->xfbHeight, vmode->fbWidth * 2);

	// Clear framebuffers etc.
	VIDEO_ClearFrameBuffer (vmode, xfb[0], COLOR_BLACK);
	VIDEO_ClearFrameBuffer (vmode, xfb[1], COLOR_BLACK);
	VIDEO_SetNextFramebuffer (xfb[0]);

	VIDEO_SetBlack (FALSE);
	VIDEO_Flush ();
	VIDEO_WaitVSync ();
	if (vmode->viTVMode & VI_NON_INTERLACE)
		VIDEO_WaitVSync ();

	StartGX();
	ResetVideo();

	WPAD_SetDataFormat(WPAD_CHAN_ALL,WPAD_FMT_BTNS_ACC_IR);
	WPAD_SetVRes(WPAD_CHAN_ALL, vmode->viWidth, vmode->viHeight);	

	m_cCamera.CalculateProjection() ;	
} 

//--------------------------------------------------------------------
void 
CFrameWork::StartGX ()
{
	GXColor background = { 0, 0, 0, 0xff };

	/*** Clear out FIFO area ***/
	memset (&gp_fifo, 0, DEFAULT_FIFO_SIZE);

	/*** Initialise GX ***/
	GX_Init (&gp_fifo, DEFAULT_FIFO_SIZE);
	GX_SetCopyClear (background, 0x00ffffff);

	GX_SetDispCopyGamma (GX_GM_1_0);
	GX_SetCullMode (GX_CULL_NONE);
} 

//--------------------------------------------------------------------
void 
CFrameWork::ResetVideo()
{
	f32 yscale;
	u32 xfbHeight;

	VIDEO_Configure (vmode);
	VIDEO_Flush();
	VIDEO_WaitVSync();
	if (vmode->viTVMode & VI_NON_INTERLACE)
		VIDEO_WaitVSync();
	else
		while (VIDEO_GetNextField())
			VIDEO_WaitVSync();

	// clears the bg to color and clears the z buffer
	GXColor background = {0, 0, 0, 255};
	GX_SetCopyClear (background, 0x00ffffff);

	yscale = GX_GetYScaleFactor(vmode->efbHeight,vmode->xfbHeight);
	xfbHeight = GX_SetDispCopyYScale(yscale);
	GX_SetScissor(0,0,vmode->fbWidth,vmode->efbHeight);
	GX_SetDispCopySrc(0,0,vmode->fbWidth,vmode->efbHeight);
	GX_SetDispCopyDst(vmode->fbWidth,xfbHeight);
	GX_SetCopyFilter(vmode->aa,vmode->sample_pattern,GX_TRUE,vmode->vfilter);
	GX_SetFieldMode(vmode->field_rendering,((vmode->viHeight==2*vmode->xfbHeight)?GX_ENABLE:GX_DISABLE));

	if (vmode->aa)
		GX_SetPixelFmt(GX_PF_RGB565_Z16, GX_ZC_LINEAR);
	else
		GX_SetPixelFmt(GX_PF_RGB8_Z24, GX_ZC_LINEAR);

	// setup the vertex descriptor
	// tells the flipper to expect direct data
	GX_ClearVtxDesc();
	GX_InvVtxCache ();
	GX_InvalidateTexAll();

	GX_SetVtxDesc (GX_VA_TEX0, GX_DIRECT);
	GX_SetVtxDesc(GX_VA_POS, GX_DIRECT);
	GX_SetVtxDesc (GX_VA_CLR0, GX_INDEX8);
	
	GX_SetVtxAttrFmt (GX_VTXFMT0, GX_VA_POS, GX_POS_XYZ, GX_F32, 0);
	GX_SetVtxAttrFmt (GX_VTXFMT0, GX_VA_CLR0, GX_CLR_RGBA, GX_RGBA8, 0);
	GX_SetVtxAttrFmt(GX_VTXFMT0, GX_VA_TEX0, GX_TEX_ST, GX_F32, 0);
	GX_SetZMode (GX_FALSE, GX_LEQUAL, GX_TRUE);
	GX_SetArray(GX_VA_CLR0, colors, 4*sizeof(u8));

	GX_SetNumChans(1);
	GX_SetNumTexGens(1);
	GX_SetTevOp (GX_TEVSTAGE0, GX_MODULATE);
	GX_SetTevOrder(GX_TEVSTAGE0, GX_TEXCOORD0, GX_TEXMAP0, GX_COLOR0A0);
	GX_SetTexCoordGen(GX_TEXCOORD0, GX_TG_MTX2x4, GX_TG_TEX0, GX_IDENTITY);

	GX_SetViewport(0,0,vmode->fbWidth,vmode->efbHeight,0,1);
	GX_SetBlendMode(GX_BM_BLEND, GX_BL_SRCALPHA, GX_BL_INVSRCALPHA, GX_LO_CLEAR);
	GX_SetAlphaUpdate(GX_TRUE);




/*
	   // load the modelview matrix into matrix memory
		GX_LoadPosMtxImm(CAMERA->m_view, GX_PNMTX0);    

      Vector   lightPos = {6,6,6} ;

		// Create light source
		GXLightObj lightObj;
		GXColor lightColour = { 255, 128, 128, 255 };
		GX_InitLightPos(&lightObj, lightPos.x, lightPos.y, lightPos.z);
		GX_InitLightColor(&lightObj, lightColour);
		//GX_InitLightDir(&lightObj, 0, 0, -1);
		//void GX_InitLightAttnK(GXLightObj *lit_obj,f32 k0,f32 k1,f32 k2) // Constant - linear - quadratic attenuation?
		//GX_InitLightDistAttn(&lightObj, 20.0f, 0.5f, GX_DA_MEDIUM);
		//GX_InitLightSpot(&lightObj, 0.0f, GX_SP_OFF);
		GX_LoadLightObj(&lightObj, GX_LIGHT0);
		
		// Set up lighting
		GX_SetNumChans(1);
		GX_SetChanCtrl(GX_COLOR0A0, TRUE, GX_SRC_REG, GX_SRC_REG, GX_LIGHT0, GX_DF_CLAMP, GX_AF_SPOT);
		GXColor amb = { 128, 128, 128, 255 };
		GXColor mat = { 128, 128, 128, 255 };
		GX_SetChanAmbColor(GX_COLOR0A0, amb);
		GX_SetChanMatColor(GX_COLOR0A0, mat);	 
      */
}

//--------------------------------------------------------------------
void 
CFrameWork::EndFrame()
{
	GX_DrawDone ();

	whichfb ^= 1; // flip framebuffer
	GX_SetZMode(GX_TRUE, GX_LEQUAL, GX_TRUE);
	GX_SetColorUpdate(GX_TRUE);
	GX_CopyDisp(xfb[whichfb],GX_TRUE);
	VIDEO_SetNextFramebuffer(xfb[whichfb]);
	VIDEO_Flush();
	
	VIDEO_WaitVSync();
}

//--------------------------------------------------------------------
void 
CFrameWork::SetFb0()
{
   GX_DrawDone ();

	whichfb = 0; // flip framebuffer
	GX_SetZMode(GX_TRUE, GX_LEQUAL, GX_TRUE);
	GX_SetColorUpdate(GX_TRUE);
	GX_CopyDisp(xfb[whichfb],GX_TRUE);
	VIDEO_SetNextFramebuffer(xfb[whichfb]);
	VIDEO_Flush();
	
	VIDEO_WaitVSync();
}

//--------------------------------------------------------------------
void 
CFrameWork::StopGX()
{
	GX_AbortFrame();
	GX_Flush();

	VIDEO_SetBlack(TRUE);
	VIDEO_Flush();
}

//--------------------------------------------------------------------
void 
CFrameWork::AddObject( CGraphicObject* pGeom) 
{
	m_lGeom.push_front( pGeom) ;
}

//--------------------------------------------------------------------
void 
CFrameWork::RemoveObject( CGraphicObject* pGeom) 
{
	m_lGeom.remove( pGeom) ;
}

//--------------------------------------------------------------------
void		
CFrameWork::SetPointer( CGraphicObject* pGeom, BOOL bAs3D) 
{
	
	if ( m_pPointer)
		delete m_pPointer ;
			
	m_pPointer = pGeom ;
	if ( !pGeom)
		return ;
	
   // remove from list	
	pGeom->SetOrient( 0, 0, 0) ;
	RemoveObject( pGeom) ;
	m_bPointer3D = bAs3D ;
}

//--------------------------------------------------------------------
void 
CFrameWork::DeleteList() 
{
	while ( !m_lGeom.empty())  
      delete (m_lGeom.front()) ;
}

//--------------------------------------------------------------------
int
CFrameWork::CreateTextureID( const u8 *PngPointer, BOOL bWithAlpha) 
{		
	if ( m_nTexture > NUM_TEXTURES)
		return 0 ;
		
	IMGCTX pngContext;
	PNGUPROP imgProp;
	int texsize;
	int width, height;
	
	pngContext = PNGU_SelectImageFromBuffer((void*)PngPointer);
	PNGU_GetImageProperties(pngContext, &imgProp);
	width = imgProp.imgWidth;
	height = imgProp.imgHeight;
	
	if ( !bWithAlpha) {
		texsize = (width * height) * 2;
		texturedata[m_nTexture] = (unsigned char *)memalign(32, texsize);	
		GX_InitTexObj( &texObj[m_nTexture], texturedata[m_nTexture], width, height, GX_TF_RGB565, GX_CLAMP, GX_CLAMP, GX_FALSE);
		PNGU_DecodeTo4x4RGB565(pngContext, width, height, texturedata[m_nTexture]);
	} else {
		texsize = (width * height) * 2 ;
		texturedata[m_nTexture] = (unsigned char *)memalign(32, texsize);	
		GX_InitTexObj( &texObj[m_nTexture], texturedata[m_nTexture], width, height, GX_TF_RGB5A3, GX_CLAMP, GX_CLAMP, GX_FALSE);
		PNGU_DecodeTo4x4RGB5A3 (pngContext, width, height, texturedata[m_nTexture], 255);	
	}
	
	DCFlushRange(texturedata[m_nTexture], texsize);		
	PNGU_ReleaseImageContext (pngContext);
	
	m_nTexture++ ;
	return m_nTexture-1 ;
} 


//--------------------------------------------------------------------
void 
CFrameWork::RenderScene() 
{
	CListGeom::iterator pItem;
   
  // do this before drawing
   for (int i=0; i<3;i++) {
     // pass 0: all 3d Obj
     // pass 1: all trasparency
     // pass 2: all screen Obj (no Zbuffer) 
      if ( i == 2)
         GX_SetZMode(GX_FALSE, GX_LEQUAL, GX_TRUE);
      else
         GX_SetZMode(GX_TRUE, GX_LEQUAL, GX_TRUE);
      for( pItem = m_lGeom.begin(); pItem != m_lGeom.end(); ++pItem) {
         if ( (*pItem)->GetNPass() ==i)
            (*pItem)->Render() ;
      }
   }
   
  // if exists mypointer, draw it			
	if ( m_pPointer)
		m_pPointer->Render() ;
      		

	EndFrame() ;
	UpdateBeforeNewFrame() ;
}
	
//--------------------------------------------------------------------
int
CFrameWork::DoPick( unsigned int uMaskID) 
{
	CListGeom::iterator pItem;
	float	dMin = 1e10;
	float	dDist ;
	int		hitCount = 0 ;
	Vector	ptRay = CAMERA->GetRayOri() ;
	Vector	vtRay = CAMERA->GetRayDir() ;
	
   // draw everything 
	m_lSel.clear() ;
	for( pItem = m_lGeom.begin(); pItem != m_lGeom.end(); ++pItem) {
		if ( (*pItem)->m_nID & uMaskID) {
			if ( (*pItem)->HitBox( ptRay, vtRay, &dDist)) {
				hitCount++ ;
			   // if nearest in front
				if ( dDist < dMin) {
					m_lSel.push_front( *pItem) ;
					dMin = dDist ;
				} 
			   // somewhereelse in the tail 	
				else {
					m_lSel.push_back( *pItem) ;
				}
			}
		}
	}	
	
	return hitCount ;
}

//--------------------------------------------------------------------
CGraphicObject*   
CFrameWork::GetPick( int n) 
{
   if (m_lSel.empty())
      return NULL ;
   return m_lSel.front() ;
}

//--------------------------------------------------------------------
void
CFrameWork::UpdateBeforeNewFrame() 
{
	WPAD_ScanPads();

	if (WPAD_ButtonsDown(0) & WPAD_BUTTON_HOME) {
	
		WPAD_Rumble( 0, 0) ;
		StopGX();
		exit(0);
	}
	
   //Draw Cursor and calculate coordinates
	ir_t  ir;
	orient_t orient;
     
	WPAD_IR(WPAD_CHAN_0, &ir);
	WPAD_Orientation(WPAD_CHAN_0, &orient); 
	m_cCamera.SetCursorPos( ir.ax, ir.ay) ;

		
	if ( m_pPointer) {
		if ( !m_bPointer3D) {
			m_pPointer->SetPosScreenPix( ir.ax, ir.ay) ;
			m_pPointer->SetOrient( 0, 0, -orient.roll) ;
		} else {
			Vector ptPos ;
			m_cCamera.GetCursorSketch( &ptPos) ;
			m_pPointer->SetPos( ptPos.x, ptPos.y, ptPos.z) ;
		}
	}	
}

//--------------------------------------------------------------------
// Wait for any button
void pressAnyButton()
{
	u32 pressed = false;
	
	printf("\nPress any button to continue...\n");
	do {
		WPAD_ScanPads();
		pressed = WPAD_ButtonsDown(0);
	} while (!pressed);
}

//--------------------------------------------------------------------
static const u32 CACHE_PAGES = 8192;
typedef enum {dev_sd, dev_usb} tDevice;
void CheckFat()
{
  static bool bInit = false ;

   if ( !bInit) {
      if (!fatInit(CACHE_PAGES, false)) { 
         printf("Unable to initialise FAT subsystem. Are there any connected devices?\n"); 
         pressAnyButton();
         exit(0);
      } 
   }  
}

//--------------------------------------------------------------------
void        
DEBUG( char* sz) 
{
return ;
   FILE *pFile;
   static bool bInit = false ;      
   
   
   if ( !bInit) {   
      CheckFat() ;
      pFile = fopen("sd:/Debug.txt", "w");
      fclose(pFile) ;
      bInit = true ;
   }
   
   pFile = fopen("sd:/Debug.txt", "a");
   fprintf( pFile, sz) ;
   fclose(pFile) ;
}
