/****************************************************************************
*   EK's ITALIAN parking                                                     *
*                                                                              *
*   17.04.09 Game.cpp                                                        *
*                                                                               *
****************************************************************************/

#include "..\include\MyModels.h"
#include "..\include\SideWalk.h"
#include "..\include\Game.h"
#include <math.h>


//------------------------------------------------------------------------------
CGame::CGame() 
{
   m_aFlags = NULL ;
   m_nRumble = -1 ;
   m_bDragging = false ;
   
   m_pTitle = NULL ;
   m_pnMoves = NULL ; 
   m_nStateGame = 0 ;
   m_nTime = 0 ;
}

//------------------------------------------------------------------------------
CGame::~CGame() 
{
   FreeAll() ;
}

//------------------------------------------------------------------------------
void
CGame::FreeAll() 
{
   if ( m_aFlags != NULL)
      free( m_aFlags) ;
   m_lElem.clear() ;
   FRAMEWORK->DeleteList() ;
   m_lExits.clear() ;
   m_lUndo.clear() ;
}

//------------------------------------------------------------------------------
void  
CGame::SetGrid( int x, int y) 
{
   int   x1, y1 ;

   FreeAll() ;
   
   m_pTitle = new CTitle ;
   m_pTitle->SetSize( 0.25, 0.25) ;
   m_pTitle->SetPosScreen( -0.5, 0.16) ;
   
   m_pnMoves = new CTextObj ;
   m_pnMoves->SetPosScreen( 0.0, -0.36) ;
   UpdateTextMove() ;

   CAMERA->SetEye(x/2.+1, -5.0F, 9.0F) ; 
   CAMERA->SetFocus(x/2.+1, y/2.+1, 1.0F) ;
   
   m_nWidth = x+2 ;
   m_nHeight = y+2 ;
   m_aFlags = (u8*) malloc(sizeof(u8)*m_nWidth*m_nHeight) ;
   for ( x1=0; x1 < m_nWidth; x1++) 
      for ( y1=0; y1 < m_nHeight; y1++) {

         if ( x1==0 || x1==m_nWidth-1) 
            SetFlagPos( x1-1, y1-1, 2) ; // LOCKPOSITION
         else if ( y1==0 || y1==m_nHeight-1) 
            SetFlagPos( x1-1, y1-1, 2) ; // LOCKPOSITION
         else {
            SetFlagPos( x1-1, y1-1, 0) ; // Free POSITION
         }
      }
}

//------------------------------------------------------------------------------
void  
CGame::AddExit( int x, int y) 
{
   TExit tExit ;
   SetFlagPos( x, y, 0) ; // Free POSITION
   
   tExit.x = x ;
   tExit.y = y ;
   m_lExits.push_front( tExit) ;
}

//------------------------------------------------------------------------------
// sideWalk
void  
CGame::CreateScene()
{
   int      x ,y ;
   int      i ;
   CTerrain* ptr  ;
   CSideWalk* ptr2  ;
   
   new CSky ;
   
   for ( x=-1; x<m_nWidth-1; x++) {
      for ( y=-1;y<m_nHeight-1; y++) {
         if ( !GetFlagPos( x, y)) {
            ptr = new CTerrain ;
            ptr->SetPos( x,y,0) ;
         } else {
            ptr2 = new CSideWalk ;
            i  = GetFlagPos( x-1, y-1) ? 0x01 : 0 ;
            i += GetFlagPos( x  , y-1) ? 0x02 : 0 ;
            i += GetFlagPos( x+1, y-1) ? 0x04 : 0 ;
            i += GetFlagPos( x-1, y  ) ? 0x08 : 0 ;
            i += GetFlagPos( x  , y  ) ? 0x10 : 0 ;
            i += GetFlagPos( x+1, y  ) ? 0x20 : 0 ;
            i += GetFlagPos( x-1, y+1) ? 0x40 : 0 ;
            i += GetFlagPos( x  , y+1) ? 0x80 : 0 ;
            i += GetFlagPos( x+1, y+1) ? 0x100 : 0 ;
            ptr2->Create( i) ;
            ptr2->SetPos( x,y,0) ;
         }
      }
   }
   
   CHouse*  pHouse ;
   
   pHouse = new CHouse() ;
   pHouse->SetPos( m_nWidth/2, +m_nHeight, 0 ) ;

   pHouse = new CHouse() ;
   pHouse->SetPos( m_nWidth, +1, 0) ;
   pHouse->SetOrient( 0,0, 90) ;
   
   
}

//------------------------------------------------------------------------------
void  
CGame::AddElement( int nType, int nX, int nY, bool bVert) 
{
   TGameElem   elem ;
   switch (nType) {
      case BLOCK_2_STD  :
         elem.pGr = new CMy147 ;
         elem.nLen = 2 ;
         ((CMy147*)elem.pGr)->SetColor(0) ;
      break ;
      case BLOCK_3_STD  :
         elem.pGr = new CBus ;
         elem.nLen = 3 ;
      break ;
      case BLOCK_2_GOAL :
         elem.pGr = new CMy147 ;
         elem.nLen = 2 ;
         ((CMy147*)elem.pGr)->SetColor(1) ;
        // save position of one goal
         SET_VECTOR( m_CarPos, nX, nY, 1) ;
      break ;
      default:
         return ;
   }
   if ( bVert) {
      elem.pGr->SetPos( nX+1, nY, 0) ;
      elem.pGr->SetOrient( 0,0, 90) ;
      elem.nDir = 1 ;
   } else {
      elem.pGr->SetPos( nX, nY, 0) ;
      elem.pGr->SetOrient( 0,0, 0) ;
      elem.nDir = 0 ;
   }

   CShadow*  pSh = new CShadow ;
   pSh->SetSource(  elem.pGr) ;
   
   AttachDetachElem( elem, 1) ;
   m_lElem.push_front( elem) ;
}

//------------------------------------------------------------------------------
void  
CGame::MainCycle( ir_t&  ir) 
{
   if ( m_nStateGame == 0)
      Intro() ;

   if ( m_nStateGame == 2)
      Outro() ;

   if ( m_nStateGame == 1) {
      ManageRotation( ir) ;
      ManageDrag( ir) ;
      UpdateRumble() ;
      TestEndGame() ;
         
      if (WPAD_ButtonsDown(0) & WPAD_BUTTON_MINUS) 
         Undo() ;
   }
}

//------------------------------------------------------------------------------
void  
CGame::Intro() 
{
   Vector pos ;
   Vector keys[10] ;

  // init splines
   if ( m_nTime == 0) {
     // position of the Eye
      keys[0] = m_CarPos ;
      keys[1] = m_CarPos ;
      keys[2] = m_CarPos ;
      keys[3] = m_CarPos ;
      keys[4] = m_CarPos ;
      keys[5] = CAMERA->Eye() ;
      keys[6] = CAMERA->Eye() ;
      m_EyePos.Init( 3, 7, keys) ;

     // position of focus
      m_CarPos.z -= 0.5 ;
      keys[0] = m_CarPos ; keys[0].x += 1 ;
      keys[1] = m_CarPos ; keys[1].y += 1 ;
      keys[2] = m_CarPos ; keys[2].x -= 1 ;
      keys[3] = m_CarPos ; keys[3].y -= 1 ;
      keys[4] = m_CarPos ; keys[4].x += 1 ;
      keys[5] = CAMERA->Focus() ;
      keys[6] = CAMERA->Focus() ;
      m_EyeFocus.Init( 4, 7, keys) ;
   }

  // switch to game
   if ( m_nTime > 298) 
      m_nStateGame = 1 ;

  // 6 position in 300/60fps
   m_EyePos.GetAt( 5*m_nTime/300.0, &pos) ;
   CAMERA->SetEye( pos.x, pos.y, pos.z) ;
      
  // 6 position in 300/60fps
   m_EyeFocus.GetAt( 5*m_nTime/300.0, &pos) ;
   CAMERA->SetFocus( pos.x, pos.y, pos.z) ;
   
   m_nTime++ ;
   
}

//------------------------------------------------------------------------------
void  
CGame::Outro() 
{
   Vector pos ;
   Vector keys[10] ;
   CListExit::iterator pItem ;

  // init splines
   if ( m_nTime == 0) {

      

     // test all     
      pItem = m_lExits.begin(); 
      SET_VECTOR( pos, (*pItem).x+0.5, (*pItem).y+0.5, 1) ;

     // position of the Eye
      keys[0] = CAMERA->Eye() ;
      SET_VECTOR( keys[1], 4, 4, 2) ;
      SET_VECTOR( keys[2], 4, 4, 1) ;
      keys[3] = pos ;
      m_EyePos.Init( 4, 4, keys) ;

     // position of focus
      m_CarPos.z -= 0.5 ;
      keys[0] = CAMERA->Focus() ;
      keys[1] = CAMERA->Focus() ;
      keys[2] = pos ;
      keys[3] = pos ;
      m_EyeFocus.Init( 4, 4, keys) ;
   }

  // switch to game
   if ( m_nTime > 299) 
      m_nStateGame = 3;

  // 3 position in 300/60fps
   m_EyePos.GetAt( 2*m_nTime/300.0, &pos) ;
   CAMERA->SetEye( pos.x, pos.y, pos.z) ;
  // 3 position in 300/60fps
   m_EyeFocus.GetAt( 2*m_nTime/300.0, &pos) ;
   CAMERA->SetFocus( pos.x, pos.y, pos.z) ;
   
   m_nTime++ ;
}

//------------------------------------------------------------------------------
void  
CGame::ManageRotation( ir_t&  ir) 
{
   static Vector  ptLast ;
   Vector ptCurr ;

   if ( !ir.raw_valid) 
      return ;
   if (WPAD_ButtonsDown(0) & WPAD_BUTTON_B) {
      CAMERA->ScreenToPoint2D( ir.ax, ir.ay, &ptLast) ;
   }
   if (WPAD_ButtonsHeld(0) & WPAD_BUTTON_B) {
      CAMERA->ScreenToPoint2D( ir.ax, ir.ay, &ptCurr) ;
      CAMERA->RotateEye( -M_PI * (ptCurr.x-ptLast.x), -M_PI * 0.5 *(ptCurr.y-ptLast.y)) ;
      ptLast = ptCurr ;
   }
}

//------------------------------------------------------------------------------
void
CGame::ManageDrag( ir_t&  ir) 
{
   static bool bOldHit = false ;
   
   bool     bHit ;   
   bool     bFound ; 
   float    dist ;
   float    dMinDist = 1e10;
   Vector   ptRay, vtRay ;
   Vector   ptSketch ;
   CListGameElem::iterator pItem, pCurrSel;

   if ( !ir.raw_valid) {
      if ( m_bDragging)
         ReleaseDrag() ;
      return ;
   }
   
   ptRay = CAMERA->GetRayOri() ;
   vtRay = CAMERA->GetRayDir() ;   
      
   bFound = FALSE ;
   for( pItem = m_lElem.begin(); pItem != m_lElem.end(); ++pItem) {
      bHit = (*pItem).pGr->HitBox( ptRay, vtRay, &dist) ;
      (*pItem).pGr->SetBlink( bHit) ;
      bFound = bFound | bHit ;
   }
   
 // when was out rumble for 5
   if ( bOldHit !=bFound) {
      if ( bFound) 
         Rumble(5) ;    // count 5 of rumble         
      else 
         Rumble( -10) ; //count 10 before another rumble
   }
   bOldHit = bFound ;   
   
   
   if (WPAD_ButtonsDown(0) & WPAD_BUTTON_A) {

     // PICK
      bFound = FALSE ;
      for( pItem = m_lElem.begin(); pItem != m_lElem.end(); ++pItem) {
         bHit = (*pItem).pGr->HitBox( ptRay, vtRay, &dist) ;
         if ( bHit && dist< dMinDist) {
            pCurrSel = pItem ;
            dMinDist = dist ;
            bFound = TRUE ;
         }
      }
      
     // BeginDrag
      if ( bFound) {
         CAMERA->GetCursorSketch( &ptSketch) ;
         BeginDrag( (*pCurrSel), ptSketch) ;
      }
   } 
   
   // DRAG   
   if ((WPAD_ButtonsHeld(0) & WPAD_BUTTON_A) && m_bDragging) {
   
      CAMERA->GetCursorSketch( &ptSketch) ;
      Drag( ptSketch) ;
   } 
   
   // RELEASE o forgetted release    
   if ( m_bDragging  && (( WPAD_ButtonsUp(0) & WPAD_BUTTON_A) || 
                        !(WPAD_ButtonsHeld(0) & WPAD_BUTTON_A))) {
      ReleaseDrag() ;
   }
 
}

//------------------------------------------------------------------------------
u8
CGame::GetFlagPos( int x, int y) 
{
   x++ ;
   y++ ;
   if ( x < 0 || x>=m_nWidth || y<0 || y>m_nHeight)
      return 8 ; // OUT_OF_BOUND

   return m_aFlags[x*m_nWidth + y] ;
}


//------------------------------------------------------------------------------
void
CGame::SetFlagPos( int x, int y, u8 nFlag) 
{
   x++ ;
   y++ ;
   if ( x < 0 || x>=m_nWidth || y<0 || y>m_nHeight)
      return ;
   m_aFlags[x*m_nWidth + y] = nFlag ;

   
}

//------------------------------------------------------------------------------
void  
CGame::TestEndGame()
{
   CListExit::iterator pItem ;

  // test all     
   for ( pItem = m_lExits.begin(); pItem != m_lExits.end(); ++pItem) 
      if ( GetFlagPos( (*pItem).x, (*pItem).y) == 0)
         return ;
  
   m_nStateGame = 2 ;   
   m_nTime = 0 ;
}

//------------------------------------------------------------------------------
void  
CGame::GetIntPos( TGameElem& elem, int* x, int* y)
{
   *x = floor(elem.pGr->m_Pos.x + 0.5) ;
   *y = floor(elem.pGr->m_Pos.y + 0.5) ;
   if ( elem.nDir == 1)
      (*x)-- ;
}

//------------------------------------------------------------------------------
void  
CGame::AttachDetachElem( TGameElem& elem, u8 nAttach) 
{
   int x, y, len ;
   
   GetIntPos( elem, &x, &y) ;
   if ( nAttach) {
      elem.pGr->SetPos( floor(elem.pGr->m_Pos.x + 0.5), floor(elem.pGr->m_Pos.y + 0.5), 0) ;
   }
   
   for ( len = 0; len <elem.nLen; len++) {
      if ( elem.nDir == 0)
         SetFlagPos( x+len,y,nAttach) ; // free ;
      else
         SetFlagPos( x,y+len,nAttach) ; // free ;
   }
}

//------------------------------------------------------------------------------
void  
CGame::GetValidMovement( TGameElem& elem, float* dMin, float* dMax)
{
   int x, y, len ;
   
   GetIntPos( elem, &x, &y) ;
   *dMin = *dMax = (elem.nDir==1) ? elem.pGr->m_Pos.y : elem.pGr->m_Pos.x ;
   
   for ( len=1; len<10; len++) {
      if ( (elem.nDir == 0 && GetFlagPos( x - len, y) == 0) ||
           (elem.nDir == 1 && GetFlagPos( x, y - len) == 0))
         *dMin = *dMin - 1 ;
      else
         break ;         
   }
   
   for ( len=elem.nLen; len<10; len++) {
      if ( (elem.nDir == 0 && GetFlagPos( x + len, y) == 0) ||
           (elem.nDir == 1 && GetFlagPos( x, y + len) == 0))
         *dMax = *dMax + 1 ;
      else
         break ;         
   }      
}

//------------------------------------------------------------------------------
// positive RUMBLE for nCicle
// Negative DONT'T RUMBLE for -nCicle
void  
CGame::Rumble( int nRumble)
{
  // don't do other rumble if not disabled...
   if ( nRumble>0 && m_nRumble >= 0) 
      m_nRumble = nRumble ;

   if ( nRumble < 0) {
      m_nRumble = nRumble ;
   }
}

//------------------------------------------------------------------------------
void  
CGame::UpdateRumble( void)
{
   if ( m_nRumble > 0) {
      WPAD_Rumble( 0, 1); // rumble on
      m_nRumble-- ;
   } 
   else if ( m_nRumble == 0) {
      WPAD_Rumble( 0, 0); // rumble off
   }
   else {
      if ( m_nRumble < 0) 
         m_nRumble++ ;
      WPAD_Rumble( 0, 0); // rumble off
   }
}

//------------------------------------------------------------------------------
//--------- DRAG OBJECTS -------------------------------------------------------
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
void  
CGame::BeginDrag( TGameElem& tElem, Vector& ptP)
{
  // state settings
   m_cCurrSel = tElem ;
   m_ptLastDrag = ptP ;
   m_bDragging = TRUE ;

  // undo settings
   m_currUndo.pGr = m_cCurrSel.pGr ;   
   m_currUndo.ptOri = m_cCurrSel.pGr->m_Pos ;

  // detach from matrix
   AttachDetachElem( m_cCurrSel, 0) ;
  // retrieve movement
   GetValidMovement( m_cCurrSel, &m_dDragMin, &m_dDragMax) ;
}

//------------------------------------------------------------------------------
void  
CGame::ClampDrag( float& x) 
{ 
   if ( x < m_dDragMin) {
      x = m_dDragMin ;
      Rumble(5) ;
   }

   if ( x > m_dDragMax) {
      x = m_dDragMax ;
      Rumble(5) ;
   }
}
//------------------------------------------------------------------------------
void  
CGame::Drag( Vector& ptNew)
{
   Vector ptPos ;

   ptPos = m_cCurrSel.pGr->m_Pos ;
   if (m_cCurrSel.nDir == 0) {
      ptPos.x += ptNew.x - m_ptLastDrag.x ;
      ClampDrag( ptPos.x) ;
   } else {
      ptPos.y += ptNew.y - m_ptLastDrag.y ;
      ClampDrag( ptPos.y) ;
   }
   m_cCurrSel.pGr->SetPos( ptPos) ;

   m_ptLastDrag = ptNew ;
}

//------------------------------------------------------------------------------
void
CGame::ReleaseDrag()
{
  // state
   m_bDragging = FALSE ;

  // put in the matrix
   AttachDetachElem( m_cCurrSel, 1) ;

  // manage undo
   m_currUndo.ptEnd = m_cCurrSel.pGr->m_Pos ;
   AddMoveToUndo( m_currUndo) ;
}

//------------------------------------------------------------------------------
//------------ UNDO ------------------------------------------------------------
//------------------------------------------------------------------------------
#define SAMEPOINT( a,b)  fabs(a.x - b.x)< 0.001 && fabs(a.y - b.y)< 0.001 && fabs(a.z - b.z)< 0.001
void  
CGame::AddMoveToUndo( TUndo& undo) 
{
   if ( ! (SAMEPOINT( m_currUndo.ptEnd, m_currUndo.ptOri))) 
      m_lUndo.push_front( m_currUndo) ;
      
   UpdateTextMove() ;
}

//------------------------------------------------------------------------------
void  
CGame::Undo( void) 
{
   CListGameElem::iterator pItem ;
   
   if ( !m_lUndo.empty()) {
   
      for ( pItem = m_lElem.begin(); pItem != m_lElem.end(); ++pItem) 
         AttachDetachElem( *pItem, 0) ;
         
      m_currUndo = m_lUndo.front() ;
      m_lUndo.pop_front() ;
      m_currUndo.pGr->SetPos( m_currUndo.ptOri) ; 
      for( pItem = m_lElem.begin(); pItem != m_lElem.end(); ++pItem) 
         AttachDetachElem( *pItem, 1) ;
   }
   UpdateTextMove() ;
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
void  
CGame::UpdateTextMove()
{
   char szBuff[20] ;
   sprintf( szBuff, "Moves: %02d", m_lUndo.size()) ;
   m_pnMoves->SetString( szBuff, 0, 0.1, 0.05) ;
}

//------------------------------------------------------------------------------
bool  
CGame::IsNotEnd() 
{ 
   return ( m_nStateGame != 3) ;
} 

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
void
SkipSpace( char* &szString)
{
   while (  szString[0] != 10 &&  szString[0] != 13 && 
            szString[0] != ' ' && szString[0] != 0)
      szString++ ;
   if ( szString != 0)
      szString++ ;
}

//------------------------------------------------------------------------------
int  
CGame::ParseLevel( int nLev, char* szString, u32 size, bool bTestOnlyExists) 
{
   char* szIdx ;
   int   n,n1,n2,n3 ;
   bool  bFound = false;
   
   szIdx = szString ;
   m_nExpectedMoves = 0 ;
   while ( szIdx < szString+size && szString[0] != 0 ) {
   
      if ( szIdx[0]=='L' && szIdx[1]=='V') {
         sscanf( &(szIdx[2]), "%d", &n) ;
         
         
         if ( n == nLev) { 
            if ( bTestOnlyExists)
               return 1 ;
            bFound = true ;
            break ;
         }
         
      }
      if ( szIdx[0] == 'E' && szIdx[1]=='N') {
         return 0 ;
      }
         
      szIdx++ ;        
   }   
   
   if ( !bFound) {
      return 0 ;
   }
   
   bFound = false ;
   SkipSpace( szIdx) ;
   
   while ( szIdx < szString+size && szString[0] != 0 ) {
      
     // Grid Dimension 
      if ( szIdx[0]=='G' && szIdx[1]=='R') {
         if ( sscanf( &(szIdx[2]), "%d,%d", &n, &n1) != 2) {
            return 0 ;
         }
         SetGrid( n, n1) ;
      }

     // Grid Dimension 
      else if ( szIdx[0]=='E' && szIdx[1]=='X') {
         if ( sscanf( &(szIdx[2]), "%d,%d", &n, &n1) != 2) {
            return 0 ;
         }
         AddExit( n, n1) ;
      }

     // Piece Type & orientation
      else if ( szIdx[0]=='P' && szIdx[1]=='I') {
         if ( sscanf( &(szIdx[2]), "%d,%d,%d,%d", &n, &n1,&n2,&n3) != 4) {
            return 0 ;
         }
         if ( !bFound)
            CreateScene() ;
         bFound = TRUE ;         
         AddElement( n, n1, n2, n3) ;
      }
      
     // Piece Type & orientation
      else if ( szIdx[0]=='M' && szIdx[1]=='V') {
         if ( sscanf( &(szIdx[2]), "%d", &m_nExpectedMoves) != 1) {
            return 0 ;
         }
         if ( !bFound)
            CreateScene() ;
         bFound = TRUE ;         
         AddElement( n, n1, n2, n3) ;
      }
      
      else if ( szIdx[0]=='E' && szIdx[1]=='N') {
         break ;
      }

      else if ( szIdx[0]=='L' && szIdx[1]=='V') {
         break ;
      }
      SkipSpace( szIdx) ;
   }
    
   
   return 1 ;
}

//------------------------------------------------------------------------------
void
CGame::WriteResult( int nLevel) 
{
   FILE *pFile ;
   int nMem = m_lUndo.size() ;
   
   CheckFat() ;
   pFile = fopen("sd:/ItalTraf.txt", "a");

   fseek( pFile, nLevel*4, SEEK_SET);
   fwrite( &nMem, 4, 1, pFile) ;

   fclose(pFile) ;   
}

//------------------------------------------------------------------------------
int
CGame::ReadResult( int nLevel) 
{
   FILE *pFile ;
   int nMem ;
   
   CheckFat() ;
   pFile = fopen("sd:/ItalTraf.txt", "r");

   fseek( pFile, nLevel*4, SEEK_SET);
   fread( &nMem, 4, 1, pFile) ;
   fclose(pFile) ;   
   
   return nMem ;
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
CChooser::CChooser(int nLevels)
{
   int   i ;
   int       nRow ;
   CTextObj* pTxt;
   char      szBuff[20] ;
   
   FRAMEWORK->DeleteList() ;
   
   nRow = nLevels / 3 + 2;
   for ( i=0; i<nLevels; i++) {
      
      pTxt = new CTextObj ;
      sprintf(szBuff, "Level %d", i+1) ;
      pTxt->SetString( szBuff, 0, 0.05, 0.025, true) ;
      pTxt->SetID( i+1) ;
      pTxt->SetPos( -0.25 + i%3 * 0.25, 0.36-(0.72/nRow)*(0.5+i/3), -1) ;      
   }
   
   pTxt = new CTextObj ;
   sprintf(szBuff, "Back") ;
   pTxt->SetString( szBuff, 0, 0.05, 0.05, true) ;
   pTxt->SetID( -1) ;
   pTxt->SetPos( -0.25 + i%3 * 0.25, 0.36-(0.72/nRow)*(0.5+i/3), -1) ;      
   
   CAMERA->SetEye( 0,0,0) ;
   CAMERA->SetFocus( 0,0, -1) ;
}

int 
CChooser::GetChoose() 
{
   if (WPAD_ButtonsDown(0) & WPAD_BUTTON_A) {
      if ( FRAMEWORK->DoPick()>0)
         return FRAMEWORK->GetPick()->GetID() ;
   }
   return 0 ;
}
