/****************************************************************************
*	EK's ITALIAN parking                                                     *
*	        																                   *
*	17.04.09 GRObject.cpp             										          *
* 	        																                   *
****************************************************************************/
#include <math.h>
#include <stdio.h>
#include "..\include\GrObject.h"
#include "..\include\FrameWork.h"

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//-----  CGraphicObject  ---------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

//--------------------------------------------------------------------
CGraphicObject::CGraphicObject() 
{
	m_nID = -1 ;
   m_nPass = 0 ;
	m_nAutoBlink = -1 ;
   m_nTrasp = 0 ;
	SET_VECTOR( m_Pos, 0, 0, 0) ;
	m_dA = 0 ; m_dB = 0; m_dC = 0 ;
	SET_VECTOR( m_ptMin, 0, 0, 0) ;
	SET_VECTOR( m_ptMax, 0, 0, 0) ;
	PrepareRef() ;
	
	FRAMEWORK->AddObject( this) ;
} ;

//--------------------------------------------------------------------
CGraphicObject::~CGraphicObject()
{
   FRAMEWORK->RemoveObject( this) ;
}

//--------------------------------------------------------------------
void  
CGraphicObject::SetID( int nID) 
{ 
	m_nID = nID ; 
} 

//--------------------------------------------------------------------
void  
CGraphicObject::SetPos( float dX, float dY, float dZ) 
{ 
	SET_VECTOR( m_Pos, dX, dY, dZ) ; 
	PrepareRef() ;
	m_bOnScreen = false ;
} ;

//--------------------------------------------------------------------
void 
CGraphicObject::SetPos( Vector& v) 
{
   m_Pos = v ;
	PrepareRef() ;
	m_bOnScreen = false ;
}

//--------------------------------------------------------------------
void  
CGraphicObject::SetOrient( float dA, float dB, float dC) 
{ 	
	m_dA = dA ; 
	m_dB = dB ; 
	m_dC = dC ; 
	PrepareRef() ;
} ;
	
//--------------------------------------------------------------------
void 
CGraphicObject::SetPosScreenPix( int WiimoteX, int WiimoteY) 
{
   // stretch with the view perspective   	
   
	CAMERA->ScreenToPoint2D( WiimoteX, WiimoteY, &m_Pos) ;
	m_dA = 0 ;
	m_dB = 0 ;
	PrepareRef() ;
	m_bOnScreen = true ;	
}

//--------------------------------------------------------------------
// -0.5..0.5 -0.5..0.5 (0,0 is the center)
// visible on Wide -0.5..0.5 -0.36..0.36 (0,0 is the center)
void 
CGraphicObject::SetPosScreen( float  dX, float dY) 
{
   // stretch with the view perspective   	
   
	SET_VECTOR( m_Pos, dX, dY, -1) ;
	m_dA = 0 ;
	m_dB = 0 ;
   m_nPass = 2 ; // on screen draw on pass 2
	PrepareRef() ;
	m_bOnScreen = true ;	
}
    
//--------------------------------------------------------------------
void 
CGraphicObject::PrepareRef() 
{
	Mtx tmp ;
   
	guMtxIdentity( m_model);	
   
	if ( fabs( m_dA) > 0.01) {
      guMtxRotAxisDeg(tmp, &stdAxisX, m_dA);
      guMtxConcat (m_model, tmp, m_model);		
	}
   
	if ( fabs( m_dB) > 0.01) {
      guMtxRotAxisDeg(tmp, &stdAxisY, m_dB);	
      guMtxConcat (m_model, tmp, m_model);
	}
   
	if ( fabs( m_dC) > 0.01) {
      guMtxRotAxisDeg(tmp, &stdAxisZ, m_dC);
      guMtxConcat (m_model, tmp, m_model);
	}
   
	guMtxTransApply(m_model, m_model, m_Pos.x, m_Pos.y, m_Pos.z);
}


//--------------------------------------------------------------------
void 
CGraphicObject::SetBlink( bool bBlink) 
{
	if ( !bBlink)
		m_nAutoBlink = -1 ;
	else if ( m_nAutoBlink < 0)
		m_nAutoBlink = 0 ;	
}

//--------------------------------------------------------------------
void 
CGraphicObject::Render() 
{
	Mtx modelview ;
	
	if ( m_bOnScreen) {
	   // no camera matrix
		GX_LoadPosMtxImm(m_model, GX_PNMTX0);    
		
	} else {
	   // camera transformation
		guMtxConcat( CAMERA->m_view, m_model, modelview);   
	   // load the modelview matrix into matrix memory
		GX_LoadPosMtxImm(modelview, GX_PNMTX0);    
	}
   // call unreferenced drawing	
	OnRender() ;
}	

//--------------------------------------------------------------------
bool
CGraphicObject::HitBox( Vector& raySrcOri, Vector& rayDirOri, float* dDist) 
{
	Mtx inv ;
	Vector rayDir ;
	Vector raySrc ;
	
	if ( m_bOnScreen) {
      rayDirOri = CAMERA->GetRayDirScreen() ;
      SET_VECTOR( raySrcOri, 0, 0, 0) ;		
   } else {
      rayDirOri = CAMERA->GetRayDir() ;
      raySrcOri = CAMERA->GetRayOri() ;
   }
	
	*dDist = -1 ;
	guMtxInverse( m_model, inv) ;
	
	rayDir.x = rayDirOri.x*inv[0][0] + rayDirOri.y*inv[0][1]+rayDirOri.z*inv[0][2] ;
	rayDir.y = rayDirOri.x*inv[1][0] + rayDirOri.y*inv[1][1]+rayDirOri.z*inv[1][2] ;
	rayDir.z = rayDirOri.x*inv[2][0] + rayDirOri.y*inv[2][1]+rayDirOri.z*inv[2][2] ;
                                                                         
	raySrc.x = raySrcOri.x*inv[0][0] + raySrcOri.y*inv[0][1]+raySrcOri.z*inv[0][2] + inv[0][3];
	raySrc.y = raySrcOri.x*inv[1][0] + raySrcOri.y*inv[1][1]+raySrcOri.z*inv[1][2] + inv[1][3];
	raySrc.z = raySrcOri.x*inv[2][0] + raySrcOri.y*inv[2][1]+raySrcOri.z*inv[2][2] + inv[2][3];
	                                                                
	float tmin = -1, tmax= -1, tymin= -1, tymax= -1, tzmin= -1, tzmax= -1, txmin=-1, txmax=-1;
	
	if ( fabs( rayDir.x) <= 1e-6) {
		if ( raySrc.x < m_ptMin.x || 
		     raySrc.x > m_ptMax.x ) {
//			printf( "\x1b[4;2H	1: %lf, %lf, %lf, %lf, %lf, %lf", txmin, txmax, tymin, tymax, tzmin, tzmax) ;
			return false ;
			}
		tmin = -1e10 ;
		tmax = +1e10 ;		
	} 
	else if ( rayDir.x >= 0) {
		tmin = ( m_ptMin.x - raySrc.x) / rayDir.x ;
		tmax = ( m_ptMax.x - raySrc.x) / rayDir.x ;
	}
	else {
		tmin = ( m_ptMax.x - raySrc.x) / rayDir.x ;
		tmax = ( m_ptMin.x - raySrc.x) / rayDir.x ;
	}
	txmin = tmin ;
	txmax = tmax ;
	
	if ( fabs( rayDir.y) <= 1e-6) {
		if ( raySrc.y < m_ptMin.y || 
		     raySrc.y > m_ptMax.y ) {
//			printf( "\x1b[4;2H	2: %lf, %lf, %lf, %lf, %lf, %lf", txmin, txmax, tymin, tymax, tzmin, tzmax) ;
			return false ;
		}
		tymin = -1e10 ;
		tymax = +1e10 ;		
	} 
	else if ( rayDir.y >= 0) {
		tymin = ( m_ptMin.y - raySrc.y) / rayDir.y ;
		tymax = ( m_ptMax.y - raySrc.y) / rayDir.y ;
	}
	else {
		tymin = ( m_ptMax.y - raySrc.y) / rayDir.y ;
		tymax = ( m_ptMin.y - raySrc.y) / rayDir.y ;
	}
	if ( (tmin > tymax) || (tymin > tmax) ) {
//		printf( "\x1b[4;2H	3: %lf, %lf, %lf, %lf, %lf, %lf", txmin, txmax, tymin, tymax, tzmin, tzmax) ;
		return false;	
	}
		
	if (tymin > tmin)
		tmin = tymin;
	if (tymax < tmax)
		tmax = tymax;
		
	if ( fabs( rayDir.z) <= 1e-6) {
		if ( raySrc.z < m_ptMin.z || 
		     raySrc.z > m_ptMax.z ){
//			printf( "\x1b[4;2H	4: %lf, %lf, %lf, %lf, %lf, %lf", txmin, txmax, tymin, tymax, tzmin, tzmax) ;
			return false ;
		}
		tzmin = -1e10 ;
		tzmax = +1e10 ;		
	} 
	else if (rayDir.z >= 0) {
		tzmin = (m_ptMin.z - raySrc.z) / rayDir.z;
		tzmax = (m_ptMax.z - raySrc.z) / rayDir.z;
	}
	else {
		tzmin = (m_ptMax.z - raySrc.z) / rayDir.z;
		tzmax = (m_ptMin.z - raySrc.z) / rayDir.z;
	}
	
	if ( (tmin > tzmax) || (tzmin > tmax) ) {
//		printf( "\x1b[4;2H	5: %lf, %lf, %lf, %lf, %lf, %lf", txmin, txmax, tymin, tymax, tzmin, tzmax) ;
		return false;
	}
		
	if (tzmin > tmin)
		tmin = tzmin;
	if (tzmax < tmax)
		tmax = tzmax;
		
	*dDist = tmin ;
//	printf( "\x1b[4;2H	6: %lf, %lf, %lf, %lf, %lf, %lf", txmin, txmax, tymin, tymax, tzmin, tzmax) ;
	
	return true ;
}

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------
CVtxObject::CVtxObject()
{
	m_nFaces = 0 ;
	m_Model_vertex = NULL ;
	m_Model_mapping = NULL ; 
	m_Model_faces = NULL ;
	m_Model_mapfaces = NULL ;
   m_nTexture = -1 ;
}


//--------------------------------------------------------------------
void 
CVtxObject::SetUp( int nTexture, int nFaces, 
        		   float Model_vertexes[][3], float Model_mapping[][2], 
				   int Model_faces[][3], int Model_mapfaces[][3]) 
{
	m_nTexture = nTexture ;
	m_nFaces = nFaces ;
	m_Model_vertex = (float*)Model_vertexes ;
	m_Model_mapping = (float*)Model_mapping ; 
	m_Model_faces = (int*)Model_faces ;
	m_Model_mapfaces = (int*)Model_mapfaces ;
	CalculateExtension() ;
}				

//--------------------------------------------------------------------
void 
CVtxObject::OnRender() 
{
	if (m_nFaces==0)
		return ;
      
	int nFace ;
	int	nVtx ;
	int	nVtxIdx ;
	int	col ;

	if ( m_nAutoBlink < 0)
		col = m_nTrasp ;
	else {
		m_nAutoBlink = (m_nAutoBlink + 1) & 0x1F ;
		col = abs(m_nAutoBlink/2-8) ;
	} 
   
   if ( m_nTexture>=0) 
   	GX_LoadTexObj(&(FRAMEWORK->texObj[m_nTexture]), GX_TEXMAP0);
      
	GX_Begin(GX_TRIANGLES, GX_VTXFMT0, m_nFaces*3);			
	for ( nFace = 0; nFace < m_nFaces; nFace++) {
	    
		for ( nVtx=0; nVtx<3; nVtx++) {
			nVtxIdx = m_Model_faces[nFace*3+nVtx] ;
			GX_Position3f32( m_Model_vertex[(nVtxIdx-1)*3], 
                          m_Model_vertex[(nVtxIdx-1)*3+1], 
                          m_Model_vertex[(nVtxIdx-1)*3+2]) ;  
         
			GX_Color1x8( col);
			    
			nVtxIdx = m_Model_mapfaces[nFace*3+nVtx] ;
			GX_TexCoord2f32( m_Model_mapping[(nVtxIdx-1)*2+0],
							 m_Model_mapping[(nVtxIdx-1)*2+1]);
		}	
			
	}
   GX_End();									
}

#define keepmin(a,b) if ( b<a) a = b ;
#define keepmax(a,b) if ( b>a) a = b ;
//--------------------------------------------------------------------
void 
CVtxObject::CalculateExtension() 
{
	int nFace ;
	int	nVtx ;
	int	nVtxIdx ;
   
	SET_VECTOR( m_ptMin, 1e8, 1e8, 1e8) ;
	SET_VECTOR( m_ptMax, -1e8, -1e8, -1e8) ;
	for ( nFace = 0; nFace < m_nFaces; nFace++) {	    
		for ( nVtx=0; nVtx<3; nVtx++) {
			nVtxIdx = m_Model_faces[nFace*3+nVtx] ;
			
			keepmin( m_ptMin.x, m_Model_vertex[(nVtxIdx-1)*3]) ;
			keepmax( m_ptMax.x, m_Model_vertex[(nVtxIdx-1)*3]) ;
			keepmin( m_ptMin.y, m_Model_vertex[(nVtxIdx-1)*3+1]) ;
			keepmax( m_ptMax.y, m_Model_vertex[(nVtxIdx-1)*3+1]) ;
			keepmin( m_ptMin.z, m_Model_vertex[(nVtxIdx-1)*3+2]) ;
			keepmax( m_ptMax.z, m_Model_vertex[(nVtxIdx-1)*3+2]) ;
		}				
	}
}
