/****************************************************************************
*	EK's ITALIAN parking                                                     *
*	        																                   *
*	17.04.09 FrameWork.cpp             										          *
* 	        																                   *
****************************************************************************/
#include "..\include\Camera.h"
#include "..\include\FrameWork.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>


//--------------------------------------------------------------------
//-----  CAMERA ------------------------------------------------------
//--------------------------------------------------------------------
CCamera::CCamera() 
{
    SET_VECTOR( m_CamEye, 0, 0, 1) ;
	SET_VECTOR( m_CamFc, 0, 0, 0) ;
	SET_VECTOR( m_ptPlane, 0, 0, 1) ;
	SET_VECTOR( m_vtPlane, 0, 0, 1) ;
}   

//--------------------------------------------------------------------
void CCamera::SetRefSketch( Vector& ptPlane, Vector& vtPlane) 
{ 
	m_ptPlane = ptPlane ; 
	m_vtPlane = vtPlane ;
}

//--------------------------------------------------------------------
void 
CCamera::SetEye( float x, float y, float z) 
{ 
	SET_VECTOR( m_CamEye, x, y, z) ; 
	CalculateProjection() ;
} 

//--------------------------------------------------------------------
void CCamera::SetFocus( float dx, float dy, float dz) 
{
	
	SET_VECTOR( m_CamFc, dx, dy, dz) ;
	
	CalculateProjection() ;
}
		 
//--------------------------------------------------------------------
void 
CCamera::CalculateProjection( void) 
{
	Vector vtZ ;
	Vector vtX ;
	Vector vtY ;

	guVecSub( &m_CamFc, &m_CamEye, &vtZ) ;	
	guVecNormalize( &vtZ) ;	
	
   // can align to Z... 	
	if ( fabs( vtZ.z) < 1-0.01) {
		guVecCross( &stdAxisZ, &vtZ, &vtX) ;
		guVecNormalize( &vtX) ;
		guVecCross( &vtZ, &vtX, &vtY) ;		
	} 
   // must align to Y... 	
	else {
		guVecCross( &stdAxisY, &vtZ, &vtX) ;
		guVecNormalize( &vtX) ;
		guVecCross( &vtZ, &vtX, &vtY) ;			
	}

	// setup our camera at the origin	
	guLookAt( m_view, &m_CamEye, &vtY, &m_CamFc);
	guMtxInverse( m_view, m_inv_view) ;

	// setup our projection matrix
	// this creates a perspective matrix with a view angle of 90,
	// and aspect ratio based on the display resolution
	if ( FRAMEWORK->GetVMode() == NULL) 
		return ;
	
	f32 w = FRAMEWORK->GetVMode()->viWidth;
	f32 h = FRAMEWORK->GetVMode()->viHeight;
	
	guPerspective( m_perspective, 45, (f32)w/h, 0.1F, 300.0F);
	GX_LoadProjectionMtx( m_perspective, GX_PERSPECTIVE);
}

//--------------------------------------------------------------------
void	
CCamera::ScreenToPoint2D( int WiimoteX, int WiimoteY, Vector* ptScreen) 
{
   // stretch with the view perspective   	
    ptScreen->x = + (((2.0f * WiimoteX) / FRAMEWORK->GetVMode()->viWidth) - 1) / m_perspective[0][0] ;
    ptScreen->y = - (((2.0f * WiimoteY) / FRAMEWORK->GetVMode()->viHeight) - 1) / m_perspective[1][1] ;
    ptScreen->z = - 1.0f ;
}

//--------------------------------------------------------------------
void
CCamera::SetCursorPos( int WiimoteX, int WiimoteY) 
{
	Vector ptScreen ;
  
	ScreenToPoint2D(  WiimoteX, WiimoteY, &ptScreen) ;
	m_vtCurrRayScreen = ptScreen  ;
   
   // rotote vector in view   
    m_vtCurrRay.x = ptScreen.x * m_inv_view[0][0] + 
	            ptScreen.y * m_inv_view[0][1] +
			    ptScreen.z * m_inv_view[0][2] ;
    m_vtCurrRay.y = ptScreen.x * m_inv_view[1][0] + 
	            ptScreen.y * m_inv_view[1][1] +
			    ptScreen.z * m_inv_view[1][2] ;
    m_vtCurrRay.z = ptScreen.x * m_inv_view[2][0] + 
			    ptScreen.y * m_inv_view[2][1] +
			    ptScreen.z * m_inv_view[2][2] ;
	
    guVecNormalize ( &m_vtCurrRay) ;
}

//--------------------------------------------------------------------
int 
CCamera::GetCursorSketch( Vector* pptRet) 
{
	Vector rayDir ;
	float dDistPlaneEye ;
	float dComp ;
	Vector vtPlaneEye ;	
	
	rayDir = GetRayDir() ;
	
	guVecSub( &m_ptPlane, &m_CamEye, &vtPlaneEye) ;
	dDistPlaneEye = guVecDotProduct( &vtPlaneEye, &m_vtPlane);
	dComp = guVecDotProduct( &rayDir, &m_vtPlane);

	if ( fabs( dComp)<1e-6) 
		return 0 ;
		
	guVecScale( &rayDir, &vtPlaneEye, dDistPlaneEye/dComp) ;
	guVecAdd( &m_CamEye, &vtPlaneEye, pptRet); 
	
	return 1 ;
}

//--------------------------------------------------------------------
void 
CCamera::RotateEye( float dH, float dV) 
{
	float dXY ;
	float dZ ;
	float dDist ;
	Vector vt ;
	
	guVecSub( &m_CamEye, &m_CamFc, &vt) ;
	dDist = sqrt( vt.x*vt.x + vt.y*vt.y + vt.z*vt.z) ;
	dXY = atan2( vt.y, vt.x) ;
	dZ = atan2( vt.z, sqrt( vt.x*vt.x + vt.y*vt.y)) ;
	
	dXY = dXY + dH ;
	dZ = dZ + dV ;
	if ( dZ < 0.1)
		dZ = 0.1 ;
	if ( dZ > M_PI/2-0.1) 
		dZ = M_PI/2-0.1 ;
		
	vt.x = dDist*cos( dXY)*cos(dZ) ;
	vt.y = dDist*sin( dXY)*cos(dZ) ;
	vt.z = dDist*sin( dZ) ;
	guVecAdd( &m_CamFc, &vt, &m_CamEye) ;
	CalculateProjection() ;
	
}
