/*********************************************************************

 Simple b-spline curve algorithm

 Copyright 1994 by Keith Vertanen (vertankd@cda.mrs.umn.edu)

 Released to the public domain (your mileage may vary)

**********************************************************************/

#include <stdlib.h>
#include <Spline.h>

//--------------------------------------------------------------------------
CSpline::CSpline()
{
   u = NULL ;
   pPoints = NULL ;
}

//--------------------------------------------------------------------------
CSpline::~CSpline()
{
   if ( u) 
      delete u;
   if ( pPoints)
      delete pPoints ;
}

//--------------------------------------------------------------------------
void compute_intervals(int *u, int n, int t)   // figure out the knots
{
  int j;

  for (j=0; j<=n+t; j++)
  {
    if (j<t)
      u[j]=0;
    else
    if ((t<=j) && (j<=n))
      u[j]=j-t+1;
    else
    if (j>n)
      u[j]=n-t+2;  // if n-t=-2 then we're screwed, everything goes to 0
  }
}

//--------------------------------------------------------------------------
void
CSpline::Init( int degree, int nPoints, Vector points[]) 
{
   n = nPoints-1 ;
   t = degree-1 ;
   
   if ( u) 
      delete u;   
   u=new int[n+t+1];

   if ( pPoints)
      delete pPoints ;

   pPoints = new Vector[nPoints] ;
   memcpy( pPoints, points, sizeof(Vector)*nPoints) ;

   compute_intervals(u, n, t);

}

//--------------------------------------------------------------------------
float 
blend(int k, int t, int *u, float v)  // calculate the blending value
{
  double value;

  if (t==1)			// base case for the recursion
  {
    if ((u[k]<=v) && (v<u[k+1]))
      value=1;
    else
      value=0;
  }
  else
  {
    if ((u[k+t-1]==u[k]) && (u[k+t]==u[k+1]))  // check for divide by zero
      value = 0;
    else
    if (u[k+t-1]==u[k]) // if a term's denominator is zero,use just the other
      value = (u[k+t] - v) / (u[k+t] - u[k+1]) * blend(k+1, t-1, u, v);
    else
    if (u[k+t]==u[k+1])
      value = (v - u[k]) / (u[k+t-1] - u[k]) * blend(k, t-1, u, v);
    else
      value = (v - u[k]) / (u[k+t-1] - u[k]) * blend(k, t-1, u, v) +
	      (u[k+t] - v) / (u[k+t] - u[k+1]) * blend(k+1, t-1, u, v);
  }
  return value;
}

//--------------------------------------------------------------------------
void 
compute_point(int *u, int n, int t, float v, Vector *control,
			Vector *output)
{
  int k;
  float temp;

  // initialize the variables that will hold our outputted point
  output->x=0;
  output->y=0;
  output->z=0;

  for (k=0; k<=n; k++)
  {
    temp = blend(k,t,u,v);  // same blend is used for each dimension coordinate
    output->x = output->x + (control[k]).x * temp;
    output->y = output->y + (control[k]).y * temp;
    output->z = output->z + (control[k]).z * temp;
  }
}

//--------------------------------------------------------------------------
void 
CSpline::GetAt( float s, Vector* p) 
{
   while ( s>n-1)
      s-=n-1 ;
   compute_point(u, n, t, s, pPoints, p);
}
