/****************************************************************************
*	EK's ITALIAN parking                                                     *
*	        																                   *
*	17.04.09 TExtObj.cpp             										          *
* 	        																                   *
****************************************************************************/

#include "..\include\textobj.h"
#include "..\include\framework.h"
#include "font_png.h"

int CTextObj::m_nTexID = -1 ;

// --------------------------------------------------------------------------
CTextObj::CTextObj() 
{
   m_nPass = 1 ;
	if ( m_nTexID < 0)
		m_nTexID  = FRAMEWORK->CreateTextureID( font_png, TRUE) ; 	
   SetUp( m_nTexID, 0, 0, 0, 0, 0) ;      
}

// --------------------------------------------------------------------------
static void
GetPos( int nFont, char nchar, float* du1, float* dv1, float* du2, float* dv2)
{

	if ( nchar < 0x20 || nchar > 0xA0)
		nchar = 0x20 ;
	
	nchar -= 0x20 ;
	if ( nFont)
		nchar += 0x80 ;
		
	*du1 = (1/16.0f)*(nchar & 0x0F) ;
	*du2 = *du1+(1/16.f)-0.004;
	*dv1 = (1/16.0f)*((nchar & 0xF0) / 16) + 0.004;
	*dv2 = *dv1+(1/16.f)-0.004;

}   
// --------------------------------------------------------------------------
void 
CTextObj::SetString( char* szString, int nFont, float dHeightText, float dWidthText, bool bCentered) 
{
	int nCount ;
	int	i ;
	char* szCurr ;
	float du1 ;
	float dv1 ;
	float du2 ;
	float dv2 ;
   float dX, dY ;
	

	if ( m_nFaces) {
		free( m_Model_vertex) ;
		free( m_Model_mapping) ;
		free( m_Model_faces) ;
		free( m_Model_mapfaces) ;
	}
	
	nCount = 0 ;
	szCurr = szString ;
	while ( szCurr[0] != 0 && nCount < 255) {
		nCount++ ;
		szCurr++ ;
	}
	
	m_nFaces = nCount*2 ;
	m_Model_vertex = (float*)malloc(sizeof(float)*3*(nCount+1)*2) ; // 2 init + 2 for char
	m_Model_mapping = (float*)malloc(sizeof(float)*2*(nCount)*4) ; // 4 coordinate (U,V) for char
	m_Model_faces = (int*)malloc(sizeof(int)*3*nCount*2) ;  // 2 triangles (3-index) for char
	m_Model_mapfaces = (int*)malloc(sizeof(int)*3*nCount*2) ; // mapping

   dX = dY = 0 ;
   if ( bCentered) {
      dX = - nCount * dWidthText * 0.5 ;
      dY = - dHeightText * 0.5 ;
   }
   // Set-Up vtx Triangles
    for ( i=0; i<=nCount; i++) {
		m_Model_vertex[ i*2*3+0] = dWidthText * i + dX;
		m_Model_vertex[ i*2*3+1] = 0. + dY;
		m_Model_vertex[ i*2*3+2] = 0. ;
		
		m_Model_vertex[ i*2*3+3] = dWidthText * i + dX ;
		m_Model_vertex[ i*2*3+4] = dHeightText + dY ;
		m_Model_vertex[ i*2*3+5] = 0. ;
	}
	
	// Set-Up vtx Faces VTX-MAP
	for ( i=0; i<nCount; i++) {
		m_Model_faces[ i*2*3+0] = i*2 +1;
		m_Model_faces[ i*2*3+1] = i*2+2 +1;
		m_Model_faces[ i*2*3+2] = i*2+1 +1;

		m_Model_faces[ i*2*3+3] = i*2+2 +1;
		m_Model_faces[ i*2*3+4] = i*2+3 +1;
		m_Model_faces[ i*2*3+5] = i*2+1 +1;
		
		m_Model_mapfaces[ i*2*3+0] = i*4 +1;
		m_Model_mapfaces[ i*2*3+1] = i*4+2 +1;
		m_Model_mapfaces[ i*2*3+2] = i*4+1 +1;
		
		m_Model_mapfaces[ i*2*3+3] = i*4+2 +1;
		m_Model_mapfaces[ i*2*3+4] = i*4+3 +1;
		m_Model_mapfaces[ i*2*3+5] = i*4+1 +1;
	}
	
   // setUp mapping 
	for ( i=0; i<nCount; i++) {
		GetPos( nFont, 	szString[i], &du1, &dv1, &du2, &dv2) ;
		m_Model_mapping[i*4*2+0] = du1 ;
		m_Model_mapping[i*4*2+1] = dv2 ;
		
		m_Model_mapping[i*4*2+2] = du1 ;
		m_Model_mapping[i*4*2+3] = dv1 ;
		
		m_Model_mapping[i*4*2+4] = du2 ;
		m_Model_mapping[i*4*2+5] = dv2 ;
		
		m_Model_mapping[i*4*2+6] = du2 ;
		m_Model_mapping[i*4*2+7] = dv1 ;
	}
     
   CalculateExtension() ;
}
